#!/usr/bin/perl

#-----------------------------------------------------------------------

#-----------------------------------------------------------------------

use strict;
use warnings;

# needed for MacOS X if using BBedit or TexWrangler (see comments below)
use File::Spec;

#------------------------------------------------------------------
# CONSTANTS
#------------------------------------------------------------------
# minimum length of motif of interest
# maximum length of motif of interest
# top-N most frequent words



# relative path of directory holding input files
my $dataDirectory = "inputGeneDirectory";		

# set-up the OUTPUT file (tab-delimited for Excel) 
my $outputFileName = "outputResults.xls";
#------------------------------------------------------------------


#------------------------------------------------------------------
# declare DATA STRUCTURES (hash table, arrays, etc)

# a hash table of frequency counts for each DNA motif
my %wordCounts;

# an array to hold each motif (L-mer)
my @allWords;

# scalar to hold each word, one at a time
my $nextWord;

my @fileList;	# all files in the directory ($dataDirectory) of data files

my $pathname;
my $filename;

my $LMER=0;		# size of motif to use in current run
#------------------------------------------------------------------


printTitle();	# print the title of the report (to the console)

my $ok = 0;  # not ok at start
while (! $ok)
{
  print "\n\n";
  # prompt user to enter a motif size



} # while not a valid LMER yet


print "Lmer = $LMER \n\n";

#------------------------------------------------------------------
# establish the absolute pathname to the directory of input files
#------------------------------------------------------------------
# MacOS USERS TAKE NOTE
# determine the absolute pathname to the current directory 
# next two lines need to be uncommented for MacOS X (e.g., BBedit or TexWrangler)
#my @path	= File::Spec->splitdir( File::Spec->rel2abs($0) );
#$pathname	= File::Spec->catfile( @path[0 .. $#path - 1], "");
#------------------------------------------------------------------
# uncomment if using
# WINDOWS or Linux (or MacOS X on command-line)
$pathname   = "";
#------------------------------------------------------------------

#------------------------------------------------------------------
# OPEN the OUTPUT file (tab-delimited for Excel) 
# Define path to output file (we will create this file).

$outputFileName = $pathname."output.xls";

open (OUTPUT, ">", $outputFileName) 
     or die "Cannot open the file: $outputFileName: $!";
#------------------------------------------------------------------


#------------------------------------------------------------------
# read data from each file in the directory
# perform stats for each file

# use filename wildcard (*) to get ALL the files in the directory
$filename = $pathname.$dataDirectory."/*";

# grab a "glob" of files, each of the filenames is put into elements of the array @fileList
@fileList = glob($filename);

my $sequence;
my $nextFile;


#==========================================================================================
# FOREACH file in the directory of input data files
    # (1) read sequence file, merge into one string
    # (2) break sequence into array of L-mers
    # (3) obtain the frequency counts of each word in the file
    # (4) SORT the words in the hash table by frequency count, high counts first
    # (5) Print <TAB>-delimited results for current file into the Excel (output) file 

foreach $nextFile (@fileList)
{
	# print the current filename to the screen (console, stdout)

	
	# print filename to Excel (output) file


    # skip Linux files with names that start with "." 
    if ($nextFile =~ m/^\./)
    {	next;  }
    
    # -----------------------------------------------------------------------------
    # (1) read sequence file, merge into one string
    $sequence = readInDNA($nextFile);	



    # -----------------------------------------------------------------------------    
    # (2) break sequence into array of L-mers
	@allWords = breakIntoMotifs($sequence, $LMER);


		
	# -----------------------------------------------------------------------------
	# (3) obtain the frequency counts of each word in the file



	# -----------------------------------------------------------------------------
 
 	# -----------------------------------------------------------------------------
    # (4) SORT the words in the hash table by frequency count, high counts first
    my @sorted;
    @sorted =  sort { $wordCounts{$b} <=> $wordCounts{$a}
                     	          or $a cmp $b
                    } keys(%wordCounts);
	# -----------------------------------------------------------------------------
	

 	# -----------------------------------------------------------------------------
    # (5) Print <TAB>-delimited results for current file into the Excel (output) file 
    printf OUTPUT "Rank\tWord\tFrequency\tProportion\n";



 	# -----------------------------------------------------------------------------
 	
 	# make some space in the Excel otuput between tables
 	print OUTPUT "\n\n\n";
 	
} # for each FILE
#==========================================================================================

close(OUTPUT);

print "\n\nDone.\n";




#--------------\
# printTitle    \
#---------------------------------------------------------------------------------
# Subroutine to print the title at the top of the output
#
# RETURNS: nothing

sub printTitle
{

} 
#----- end of printTitle ---------------------------------------------------------


#----------\
# readInDNA \
#---------------------------------------------------------------------------------
# Subroutine to open a FASTA formatted file of DNA sequence, remove
# newline characters at the end of each line, discard any blank lines
# discard lines containing a comment(#) symbol, removes any digits,
# removes any extra whitespace characters, converts nucleotides
# to uppercase, and return as one big string.
#
# 1 argument:  name of file holding the DNA
#
# RETURNS:  file of DNA as one string
#
# A sample of how you might "call" this subroutine from your program
# to 'get' the DNA from a Fasta formatted file for a gene sequence
#
#                  $sequence = readInDNA("humanXM_012345.fna");

sub readInDNA
{
	my $filename = shift;
	my $line;
	
	my $firstline;
	my $seq;
	
    # try to open the file; if no file by that name, print a warning and exit
    open (FNAFILE, "<", $filename)
         or die "Cannot open the input file: $filename: $!";
         
    if ( !($firstline = <FNAFILE>) ) {
       print "Can NOT read the header line from the file called: $filename \n";
       exit();
     }
	
	$seq = "";
  
  	# WHILE not EOF, grab next line
	while ($line = <FNAFILE>)		
	{
			chomp $line;
			#print "BEFORE: $line \n";
			
	    	# discard blank line
        	if ($line =~ m/^\s*$/) 
        	{	next; }
        	 
        	# discard comment line (any line with a comment(#) symbol)
        	if ($line =~ m/^\s*#/) 
        	{   next; } 
		
			$line =~ tr/0123456789//d;	# remove all digits
			$line =~ tr/ \t\n\r//d;		# remove any extra whitespace
			$line = uc($line);			# convert everything to upper-case
			
			#print "AFTER: $line \n";
			
    		$seq = $seq . $line;	# concatenate new line onto the entire text so far
	
	} # end WHILE not EOF
	
	close(FNAFILE);
	
	return $seq;
	
} # end subroutine readInDNA
#---------------------------------------------------------------------------------------



#----------------\
# breakIntoMotifs \
#---------------------------------------------------------------------------------------
# (comments go here)
#
#---------------------------------------------------------------------------------------
sub breakIntoMotifs
{

	my $text = shift;
	my $LMER = shift;
	
    my $DNAlen = length($text);
	
	my @words;  # array of words to fill and return when done
	
	my $i=0;	# index i points to start of the next word each time
	my $j=0;    # j indexes into the array @words; $words[$j] is the next word
    my $end = $DNAlen-($LMER-1);
	
	# a bunch of work to do here ....
   	#while (     )
    #{
	  
	    # (1) use substr() to grab the next motif/word;  store in $words[$j]
	    # (2) store in $words[$j]
	    # (3) make sure the motif-word does not contain any Ns
	    # (3a) if it does contain an 'N', skip down the $text beyond the N
	    # (3b) if not, increment indices $i and $j
	  
	  
	  
	  

    #} # while more words
	
	return @words;

} # end subroutine breakIntoMotifs
#-----------------------------------------------------------------------------------------
